Package: scRepertoire
Title: A toolkit for single-cell immune receptor profiling
Version: 2.9.0
Authors@R: c(
    person(given = "Nick", family = "Borcherding", role = c("aut", "cre"), email = "ncborch@gmail.com"), 
    person(given = "Qile", family = "Yang", role = c("aut"), email = "qile.yang@berkeley.edu"), 
    person(given = "Ksenia", family = "Safina", role = c("aut"), email = "safina@broadinstitute.org"),
    person(given = "Justin", family = "Reimertz", role = c("ctb"), email = "justin.reimertz@childrens.harvard.edu"))
Description: scRepertoire is a toolkit for processing and analyzing single-cell T-cell receptor (TCR) and immunoglobulin (Ig). The scRepertoire framework supports use of 10x, AIRR, BD, MiXCR, TRUST4, and WAT3R single-cell formats. The functionality includes basic clonal analyses, repertoire summaries, distance-based clustering and interaction with the popular Seurat and SingleCellExperiment/Bioconductor R single-cell workflows.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
biocViews: Software, ImmunoOncology, SingleCell, Classification,
        Annotation, Sequencing
Depends: ggplot2, R (>= 4.0)
Imports: dplyr, evmix, ggalluvial, ggdendro, ggraph, grDevices, igraph,
        immApex, iNEXT, Matrix, quantreg, Rcpp, rjson, rlang,
        S4Vectors, SeuratObject, SingleCellExperiment,
        SummarizedExperiment, tidygraph, purrr, lifecycle, methods
Suggests: BiocManager, BiocStyle, circlize, knitr, Peptides, rmarkdown,
        scales, scater, Seurat, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Language: en-US
LinkingTo: Rcpp
URL: https://www.borch.dev/uploads/scRepertoire/
BugReports: https://github.com/BorchLab/scRepertoire/issues
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/scRepertoire
git_branch: devel
git_last_commit: 7b864c4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 01:06:49 UTC; biocbuild
Author: Nick Borcherding [aut, cre],
  Qile Yang [aut],
  Ksenia Safina [aut],
  Justin Reimertz [ctb]
Maintainer: Nick Borcherding <ncborch@gmail.com>
