# Generated by roxygen2: do not edit by hand

S3method(quietBCRgenes,Seurat)
S3method(quietBCRgenes,default)
S3method(quietTCRgenes,Seurat)
S3method(quietTCRgenes,default)
export(StartracDiversity)
export(addVariable)
export(alluvialClones)
export(annotateInvariant)
export(clonalAbundance)
export(clonalBias)
export(clonalBin)
export(clonalCluster)
export(clonalCompare)
export(clonalDiversity)
export(clonalHomeostasis)
export(clonalLength)
export(clonalNetwork)
export(clonalOccupy)
export(clonalOverlap)
export(clonalOverlay)
export(clonalProportion)
export(clonalQuant)
export(clonalRarefaction)
export(clonalScatter)
export(clonalSizeDistribution)
export(combineBCR)
export(combineExpression)
export(combineTCR)
export(createHTOContigList)
export(exportClones)
export(expression2List)
export(getCirclize)
export(getContigDoublets)
export(highlightClones)
export(loadContigs)
export(percentAA)
export(percentGeneUsage)
export(percentGenes)
export(percentKmer)
export(percentVJ)
export(positionalEntropy)
export(positionalProperty)
export(quietBCRgenes)
export(quietTCRgenes)
export(quietVDJgenes)
export(subsetClones)
export(vizCirclize)
export(vizGenes)
import(ggplot2)
importFrom(Matrix,sparseMatrix)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,DataFrame)
importFrom(SeuratObject,Embeddings)
importFrom(SeuratObject,Idents)
importFrom(SingleCellExperiment,colData)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,colData)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,coalesce)
importFrom(dplyr,count)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,select)
importFrom(dplyr,slice_max)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,summarize_all)
importFrom(dplyr,ungroup)
importFrom(evmix,dgpd)
importFrom(evmix,fgpd)
importFrom(evmix,pgpd)
importFrom(evmix,qgpd)
importFrom(evmix,rgpd)
importFrom(ggalluvial,StatStratum)
importFrom(ggalluvial,geom_alluvium)
importFrom(ggalluvial,geom_flow)
importFrom(ggalluvial,geom_stratum)
importFrom(ggalluvial,to_lodes_form)
importFrom(ggdendro,dendro_data)
importFrom(ggdendro,label)
importFrom(ggdendro,segment)
importFrom(ggplot2,ggplot)
importFrom(ggraph,circle)
importFrom(ggraph,geom_edge_bend)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,ggraph)
importFrom(ggraph,guide_edge_colourbar)
importFrom(ggraph,scale_edge_colour_gradientn)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,hcl.pals)
importFrom(graphics,plot)
importFrom(iNEXT,ggiNEXT)
importFrom(iNEXT,iNEXT)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,`V<-`)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_data_frame)
importFrom(igraph,cluster_edge_betweenness)
importFrom(igraph,cluster_fast_greedy)
importFrom(igraph,cluster_leiden)
importFrom(igraph,cluster_louvain)
importFrom(igraph,cluster_spinglass)
importFrom(igraph,cluster_walktrap)
importFrom(igraph,components)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,graph_from_edgelist)
importFrom(igraph,membership)
importFrom(igraph,set_vertex_attr)
importFrom(immApex,ace_richness)
importFrom(immApex,buildNetwork)
importFrom(immApex,calculateEntropy)
importFrom(immApex,calculateFrequency)
importFrom(immApex,calculateGeneUsage)
importFrom(immApex,calculateMotif)
importFrom(immApex,chao1_richness)
importFrom(immApex,d50_dom)
importFrom(immApex,getIR)
importFrom(immApex,gini_coef)
importFrom(immApex,gini_simpson)
importFrom(immApex,hill_q)
importFrom(immApex,inv_simpson)
importFrom(immApex,norm_entropy)
importFrom(immApex,pielou_evenness)
importFrom(immApex,shannon_entropy)
importFrom(lifecycle,deprecated)
importFrom(methods,is)
importFrom(methods,slot)
importFrom(quantreg,rqss)
importFrom(rjson,fromJSON)
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,sym)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,hclust)
importFrom(stats,integrate)
importFrom(stats,mad)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,pgamma)
importFrom(stats,qgamma)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,reshape)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,xtabs)
importFrom(tidygraph,activate)
importFrom(tidygraph,as_tbl_graph)
importFrom(utils,combn)
importFrom(utils,getFromNamespace)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,stack)
importFrom(utils,write.csv)
useDynLib(scRepertoire, .registration = TRUE)
