#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom lifecycle deprecated
#' @importFrom Rcpp sourceCpp
#' @useDynLib scRepertoire, .registration = TRUE
#'
#' @import ggplot2
#' @importFrom dplyr %>% select mutate filter arrange bind_rows group_by n
#' @importFrom dplyr summarise summarize
## usethis namespace: end
NULL
