#' A List of Eight Single-cell TCR Sequencing Runs.
#' 
#' @description A list of 8 `filtered_contig_annotations.csv` files
#' outputted from 10X Cell Ranger. More information on the 
#' data can be found in the following
#'  [manuscript](https://pubmed.ncbi.nlm.nih.gov/33622974/).
#'  
#' @docType data
#' @concept Data
#' @name contig_list
#' 
NULL

#' A Seurat Object of 500 Single T cells,
#' 
#' @description The object is compatible with `contig_list` and the TCR
#' sequencing data can be added with `combineExpression`.  The data is 
#' from 4 patients with acute respiratory distress, with samples taken
#' from both the lung and peripheral blood. More information on the 
#' data can be found in the following
#'  [manuscript](https://pubmed.ncbi.nlm.nih.gov/33622974/).
#' 
#' @name scRep_example
#' @concept Data
#' @docType data
#'
NULL


