# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rcppConstructBarcodeIndex <- function(conDfBarcodes, data2Barcodes) {
    .Call(`_scRepertoire_rcppConstructBarcodeIndex`, conDfBarcodes, data2Barcodes)
}

rcppConstructConDfAndParseTCR <- function(data2, uniqueData2Barcodes) {
    .Call(`_scRepertoire_rcppConstructConDfAndParseTCR`, data2, uniqueData2Barcodes)
}

